// Wrapper for gVim to open up files via Unity3d Editor
// Author: William Sykes
#include <windows.h>
#include <iostream>
#include <string>
using namespace std;

define OPEN_IN_TAB

#pragma comment(lib,"Shell32.lib")
#pragma comment(lib,"Advapi32.lib")

bool getVimPath(string &out)
{
    out = "";

    HKEY hKey;
    if( RegOpenKeyEx(
            HKEY_CLASSES_ROOT,
            "TypeLib\\{0F0BFAE0-4C90-11D1-82D7-0004AC368519}\\1.1\\0\\win32",
            0L, KEY_READ, &hKey) == ERROR_SUCCESS )
    {
        char szValue[256];
        DWORD dwSize = 256;
        DWORD dwType = REG_SZ;
        if( RegQueryValueEx(hKey,NULL,NULL,&dwType,(LPBYTE)szValue,&dwSize) == 
                ERROR_SUCCESS )
        {
            out = szValue;
        }
    }
    RegCloseKey(hKey);
    
    return out.compare("") != 0;
}

int main(int argc, char **argv)
{
    /* 
    When Unity3d Editor calls the application (when it
    is renamed to MonoDevelop.exe) it will pass the following
    arguments:
    argv[1] --nologo
    argv[2] <path to MonoDevelop solution file>
    argv[3] <path to file to open>;<line number>  (-1 if line number not specified)
    */
    if( argc < 4 )
        return -1;

    string fileinfo(argv[3]);
    // To deal with directories with spaces we need to combine any remaining arguments
    for( int i = 4; i < argc; i++ )
    {
        fileinfo += " ";
        fileinfo += argv[i];
    }
    string gvimArgs = 
#ifdef OPEN_IN_TAB
        "--remote-tab-silent \"";
#else
        "--remote-silent \"";
#endif
    size_t pos = fileinfo.rfind(";-1"); 
    if( pos == string::npos )
    {
        pos = fileinfo.rfind(";");
        // A line number was specified
        fileinfo.replace(pos,1,"\":");
    }
    else
    {
        // No line number, so just open the file.
        // Chop off the ;-1 part of the argument.
        fileinfo.resize(fileinfo.length() - 3);
        fileinfo += "\"";
    }
    gvimArgs += fileinfo;
 
    string gvimPath;
    if( !getVimPath(gvimPath) )
        return -1;
    
    ShellExecute(NULL,NULL,gvimPath.c_str(),gvimArgs.c_str(),NULL,0);
    return 0;
}
